<?php
/**
 * ownCloud
 *
 * @author Owen Winkler
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

if(!defined('OC_SUPPORT_ENDPOINT')) {
	define('OC_SUPPORT_ENDPOINT', 'https://owncloud.com/support-request');
}

\OCP\JSON::checkAdminUser();
\OCP\JSON::checkAppEnabled('enterprise_key');
\OCP\JSON::callCheck();

$l = OC_L10N::get('enterprise_key');

$error = false;


// Basic report data
$userids = \OCP\User::getUsers();
$homecount = 0;
foreach($userids as $uid) {
	if(OC_User::getManager()->get($uid)) {
		$homecount++;
	}
}
$report = array(
	'basic' => array(
		'license key' => \OCP\Config::getSystemValue('license-key'),
		'date' => date('r'),
		'ownCloud version' => implode('.', OC_Util::getVersion()),
		'ownCloud version string' => OC_Util::getVersionString(),
		'ownCloud edition' => OC_Util::getEditionString(),
		'server OS' => PHP_OS,
		'server OS version' => php_uname(),
		'server SAPI' => php_sapi_name(),
		'webserver version' => $_SERVER['SERVER_SOFTWARE'],
		'hostname' => $_SERVER['HTTP_HOST'],
		'user count' => count($userids),
		'user directories' => $homecount,
		'logged-in user' => \OCP\User::getDisplayName(),
	)
);

// Get app data
$apps = OC_App::listAllApps();
foreach($apps as &$app) {
	if($app['active']) {
		$app['appconfig'] = OC_Appconfig::getValues($app['id'], false);
	}
}
$report['apps'] = $apps;

// Get the OC config and filter out the sensitive bits
$skipKeys = array_flip(array('passwordsalt', 'dbpassword'));
$keys = OC_Config::getKeys();
foreach ($keys as $key) {
	if (isset($skipKeys[$key])) {
		$report['config'][$key] = 'xxxx Value is set but was removed by reporting app for security reasons xxxx';
		continue;
	}
	$report['config'][$key] = OC_Config::getValue($key);
}

// Get the phpinfo, parse it, and record it (parts from http://www.php.net/manual/en/function.phpinfo.php#87463)
ob_start();
phpinfo(-1);

$phpinfo = preg_replace(
	array('#^.*<body>(.*)</body>.*$#ms', '#<h2>PHP License</h2>.*$#ms',
		'#<h1>Configuration</h1>#', "#\r?\n#", "#</(h1|h2|h3|tr)>#", '# +<#',
		"#[ \t]+#", '#&nbsp;#', '#  +#', '# class=".*?"#', '%&#039;%',
		'#<tr>(?:.*?)" src="(?:.*?)=(.*?)" alt="PHP Logo" /></a>'
		. '<h1>PHP Version (.*?)</h1>(?:\n+?)</td></tr>#',
		'#<h1><a href="(?:.*?)\?=(.*?)">PHP Credits</a></h1>#',
		'#<tr>(?:.*?)" src="(?:.*?)=(.*?)"(?:.*?)Zend Engine (.*?),(?:.*?)</tr>#',
		"# +#", '#<tr>#', '#</tr>#'),
	array('$1', '', '', '', '</$1>' . "\n", '<', ' ', ' ', ' ', '', ' ',
		'<h2>PHP Configuration</h2>' . "\n" . '<tr><td>PHP Version</td><td>$2</td></tr>' .
		"\n" . '<tr><td>PHP Egg</td><td>$1</td></tr>',
		'<tr><td>PHP Credits Egg</td><td>$1</td></tr>',
		'<tr><td>Zend Engine</td><td>$2</td></tr>' . "\n" .
		'<tr><td>Zend Egg</td><td>$1</td></tr>', ' ', '%S%', '%E%'),
	ob_get_clean());

$sections = explode('<h2>', strip_tags($phpinfo, '<h2><th><td>'));
unset($sections[0]);

$report['phpinfo'] = array();
foreach ($sections as $section) {
	$n = substr($section, 0, strpos($section, '</h2>'));
	preg_match_all(
		'#%S%(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?(?:<td>(.*?)</td>)?%E%#',
		$section, $matches, PREG_SET_ORDER);
	foreach ($matches as $match) {
		if(!isset($match[3])) {
			$value = isset($match[2]) ? $match[2] : null;
		}
		elseif($match[2] == $match[3]) {
			$value = $match[2];
		}
		else {
			$value = array_slice($match, 2);
		}
		$report['phpinfo'][$n][$match[1]] = $value;
	}
}

if(isset($_GET['download']) && $_GET['download'] == 1) {
	header('content-type: text/plain');
	header('content-disposition: attachment; filename=config_report_' . date('Ymd') . '.txt');
	$settings = defined('JSON_PRETTY_PRINT') ? JSON_PRETTY_PRINT : 0;
	echo json_encode($report, $settings);
}
else {
	// Post the report data to our report endpoint
	if(function_exists('curl_init')) {
		$curl = curl_init(OC_SUPPORT_ENDPOINT);

		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($report));
		curl_setopt($curl, CURLOPT_HEADER, 0);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curl, CURLOPT_URL, OC_SUPPORT_ENDPOINT);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($curl, CURLOPT_MAXREDIRS, 10);

		curl_setopt($curl, CURLOPT_USERAGENT, "ownCloud Server Crawler");
		if(OC_Config::getValue('proxy', '') != '') {
			curl_setopt($curl, CURLOPT_PROXY, OC_Config::getValue('proxy'));
		}
		if(OC_Config::getValue('proxyuserpwd', '') != '') {
			curl_setopt($curl, CURLOPT_PROXYUSERPWD, OC_Config::getValue('proxyuserpwd'));
		}
		$data = curl_exec($curl);
		if($data) {
			$data = json_decode($data);
			if(is_null($data)) {
				$error = $l->t('An invalid response was received while sending the report to the support server.') . ' (1)';
			}
			elseif(!$data->success) {
				$error = $data->message;
			}
		}
		else {
			$error = $l->t('This server could not contact ownCloud support, please select the "Download ownCloud config report" button and email the file to support@owncloud.com.');
		}
		curl_close($curl);

	}
	else {
		$error = $l->t('cURL is not available/enabled to transmit report data.');
	}

	if (!$error) {
		\OCP\JSON::success(array('data' => array('message' => $l->t('The config report was sent successfully to ownCloud support.'))));
	}
	else {
		\OCP\JSON::error(array('data' => array('message' => $error)));
	}
}

